// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <atomic>	 // for atomic_uint, atomic

#include "PrivateSemaphore.hpp"	 // for PrivateSemaphore

class Context;

/**
 * A counting private semaphore.
 */
class CountingPrivateSemaphore : public PrivateSemaphore {
 private:
	std::atomic_uint counter;
	std::atomic<Context*> blockedContext;

	inline auto signalInternal() -> Context* override;

 public:
	CountingPrivateSemaphore();
	explicit CountingPrivateSemaphore(unsigned int counter);

	inline auto getCounter() -> unsigned int { return counter.load(); };

	void incrementCounterByOne();

	void incrementCounter(unsigned int count);

	void wait() override;
};

using CPS = CountingPrivateSemaphore;
