// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#include "FiberManager.hpp"

#include <cstdlib>	// for abort, free, posix_memalign

#include "Fiber.hpp"	// for Fiber

class Runtime;

FiberManager::FiberManager(Runtime& runtime) : MemoryManager(runtime) {}

auto FiberManager::getFiberMemory() -> void* {
#ifdef FM_WITH_MEMORY_MANAGER
	bool malloced;
	return getMemory(&malloced);
#else
	void* mem;
	int res = posix_memalign(&mem, alignof(Fiber), sizeof(Fiber));
	if (res != 0) {
		abort();
	}
	return mem;
#endif	// FM_WITH_MEMORY_MANAGER
}

void FiberManager::putFiberMemory(Fiber* fiber) {
#ifdef FM_WITH_MEMORY_MANAGER
	putMemory(fiber);
#else
	free(fiber);
#endif	// FM_WITH_MEMORY_MANAGER
}
