// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include "Blockable.hpp"
#include "CallerEnvironment.hpp"
#include "Context.hpp"
#include "ContextManager.hpp"
#include "Debug.hpp"
#include "Dispatcher.hpp"
#include "Fiber.hpp"
#include "Runtime.hpp"

class PrivateSemaphore : protected Blockable<LogSubsystem::PS> {
 protected:
	// cppcheck-suppress uninitMemberVar
	PrivateSemaphore() : Blockable(*Runtime::getRuntime()) {
		LOGD("constructed by fiber " << Dispatcher::getCurrentFiber());
	}

	[[noreturn]] void unblockAndExit(Context* context) { contextManager.discardAndResume(context); }

	virtual auto signalInternal() -> Context* = 0;

	virtual void debugLog(const std::string& string) const { Logger::logD(string); }

 public:
	virtual void wait() = 0;

	template <CallerEnvironment callerEnvironment = CallerEnvironment::EMPER>
	void signal() {
		if (Context* readyContext = signalInternal()) {
			unblock<callerEnvironment>(readyContext);
		}
		// Assure that nothing happens after here with the sync
		// primitive, as the data of the sync primitive may became
		// invalid at this point.
	}

	void signalFromAnywhere() { signal<CallerEnvironment::ANYWHERE>(); }

	void signalAndExit() {
		if (Context* readyContext = signalInternal()) {
			unblockAndExit(readyContext);
		}
		// Assure that nothing happens after here with the sync
		// primitive, as the data of the sync primitive may became
		// invalid at this point.
	}
};

using PS = PrivateSemaphore;
