// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <atomic>

#include "Semaphore.hpp"

class Latch {
 private:
	const unsigned int num;
	std::atomic<unsigned int> counter;
	emper::lib::sync::Semaphore semaphore;

 public:
	Latch(unsigned int counter) : num(counter), counter(counter), semaphore(0) {}

	inline void count_down_and_wait() {
		unsigned int current = counter.fetch_sub(1) - 1;
		if (current == 0) {
			semaphore.notify_many(num);
		}
		semaphore.wait();
	}
};
