// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Fischer
#include <liburing.h>

#include <cassert>	// for assert
#include <cerrno>		// for ETIME
#include <cstdint>	// for int32_t

#include "BinaryPrivateSemaphore.hpp"
#include "io/Future.hpp"	// for AlarmFuture

using emper::io::AlarmFuture;
using emper::io::Future;

void callback(int32_t res, BPS& bps) {
	assert(res == -ETIME);
	bps.signal();
}

void emperTest() {
	struct __kernel_timespec ts = {.tv_sec = 1, .tv_nsec = 0};
	AlarmFuture alarm(ts);
	BPS bps;

	alarm.setCallback(Future::Callback([&bps](int32_t res) { callback(res, bps); }));
	alarm.submit();

	// wait till the callback was executed
	bps.wait();
}
